"""
default exception
"""
je_editor_error: str = "JEditor error"
"""
init exception
"""
je_editor_init_error: str = "JEditor can't init"
je_editor_init_exec_manager_exception: str = "JEditor can't init program manager"
"""
exec exception
"""
je_editor_exec_error: str = "JEditor exec error"
file_not_fond_error: str = "File not found"
compiler_not_found_error: str = "Compiler not found"
"""
shell exception
"""
je_editor_shell_error: str = "JEditor run on shell error"
"""
file exception
"""
je_editor_save_file_error: str = "JEditor save file error"
je_editor_open_file_error: str = "JEditor open file error"
"""
json exception
"""
cant_reformat_json_error: str = "Can't reformat json is type right?"
wrong_json_data_error: str = "Can't parser json"
cant_find_json_error: str = "cant find json file"
cant_save_json_error: str = "cant save json file"
"""
content data error
"""
content_set_compiler_error: str = "When set compiler using content make an error"
je_editor_content_file_error: str = "JEditor content file error"
